/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/

package de.hendriklipka.util;

import java.net.*;
import org.apache.log4j.Category;

/**
  An UDPSocketPool is able to create a DatagramSocket, which is connected to a 
  free local port. It searches through the port from 1024-65535 and returns
  a new DatagramSocket for the next free port.
  The strategy is to cycle through all ports to return a socket for the next free port
*/
public class UDPSocketPool extends java.lang.Object
{
    static Category CAT = Category.getInstance(UDPSocketPool.class.getName());
    
    static final byte PORT_NOT_FREE=0;
    static final byte PORT_FREE=1;
    static final byte PORT_USED=2;
    
    static byte[] PortUsed=new byte[65536];
    static int FirstFreeSocket=1024;
    static int CurrentFreeSocket=FirstFreeSocket;
    
    // ports under 1024 are in use    
    static
    {
        for (int i=0;i<FirstFreeSocket;i++)
            PortUsed[i]=PORT_NOT_FREE;
        for (int i=FirstFreeSocket;i<65536;i++)
            PortUsed[i]=PORT_FREE;
    }
    
    /**
    Creates a new socket for the next free port. It cycles through all available
    ports (1024-65535) and returns a socket for the next free port. If port 65535
    is reached, it starts again with port 1024.
    @return a new socket for the next free ports or null if no free ports are found
    */
    public static DatagramSocket getSocket()
    {
        int StartSocket=CurrentFreeSocket;
        while (true)
        {
            if (PORT_FREE==PortUsed[CurrentFreeSocket])
            {
                try
                {
                    DatagramSocket ds=new DatagramSocket(CurrentFreeSocket);
                    CurrentFreeSocket++;
                    if (65536==CurrentFreeSocket)
                        CurrentFreeSocket=FirstFreeSocket;
                    return ds;
                }
                catch (SocketException ex)
                {
                    PortUsed[CurrentFreeSocket++]=PORT_NOT_FREE;
                    if (65536==CurrentFreeSocket)
                        CurrentFreeSocket=FirstFreeSocket;
                    if (StartSocket==CurrentFreeSocket)
                        return null;
                    continue;
                }
            }
            else
            {
                CurrentFreeSocket++;
                if (65536==CurrentFreeSocket)
                    CurrentFreeSocket=FirstFreeSocket;
                if (StartSocket==CurrentFreeSocket)
                    return null;
                continue;
            }
        }
    }
    
    /**
    Free an used port for using with another Socket.
    */
    public static void freeSocket(DatagramSocket ds)
    {
        PortUsed[ds.getLocalPort()]=PORT_FREE;
    }
}
